/*
 * \file FBF.h
 * \brief FBF utility function declarations for GIPS framework
 *
 * Someday, this will grow into a full-featured FBF object interface,
 * a la FBF.py.  We await that day with baited breath.
 * \author Maciek Smuga-Otto <maciek@ssec.wisc.edu>
 *
 *  \version $Id: FBF.h,v 1.3 2004/10/31 05:07:39 maciek Exp $
 *
 *  \par Copyright:
 *  \verbatim
 *
 *  Copyright UW/SSEC, ALL RIGHTS RESERVED, 2004
 *  Space Science and Engineering Center
 *  University of Wisconsin - Madison, USA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *  \endverbatim
 */


#ifndef H_FBF
#define H_FBF

#ifndef NO_DEPENDENCY_INCLUDES

#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>

#include <string>

#endif

// FBF filename utilities

namespace FBF 
{
    
const char *nativeFilename( char *buffer );

std::string nativeFilename( const std::string &str ); ///< convert filename to native byte order encoding

int firstFilename( std::string &out, const char *pattern ); ///< match a pattern using a glob, return first match

const char *filename( const char *path ); ///< return filename at the end of a pathname 



// FBF file read/write utilities

/* load a flat binary file in its entirety into a vector */
template < typename V > 
    int load( V &out, FILE *fp, bool autoclose = false, size_t group_size = 16384 ) 
{ 
        typedef typename V::value_type M;
        // FIXME: this should use the suffix to determine casting from file data to output data
        //        rather than assume that the type of the vector and the file match
        //        further, we should consider using mmap() if we only want a read-only vector..
        if (!fp) return -1;
        size_t nread = 0, result = group_size;
        while( !feof(fp) && (result==group_size) ) 
        { /* eat the file in chunks */
            out.resize( nread + group_size );
            result = fread( &out[nread], sizeof(M), group_size, fp );
            nread += result;
        }
        out.resize( nread );
        if (autoclose) 
            fclose(fp);
        return (nread==0)? 1 : 0;
}

template< typename V >
int loadFirst( V &out, std::string &filename, const char *pattern ) 
{     
    int rc = firstFilename( filename, pattern );
    //RCHK( rc, "could not find file matching ", pattern );
    if (rc<0) return rc;
    
    rc = load( out, fopen( filename.c_str(), "rb" ), true );
    return rc;
}


template< typename M >
int save( const char *filename, const M &data, int flags=0, int mode=0664 )
{ /* save an STL vector to a given file */
int fd = 0;
fd = open( filename, O_WRONLY | O_CREAT | flags, mode );
//WHENTESTING( printf( "creating %s..\n", filename ) );
if (fd<0) return fd;
ssize_t block = (ssize_t)sizeof(typename M::value_type) * data.size();
//WHENTESTING( printf( "fd: %d..\n", fd ) );
ssize_t writ = write( fd, &data[0], block );
close(fd);
//WHENTESTING( printf( "wrote %ld of %ld bytes..\n", (long)writ, (long)block ) );
return writ==block? 0 : -1;
}

template< typename M >
int blitz_save( const char *filename, const M &data, int flags=0, int mode=0664 )
{ /* save a blitz array or matrix to a given file */
int fd = 0;
fd = open( filename, O_WRONLY | O_CREAT | flags, mode );
//WHENTESTING( printf( "creating %s..\n", filename ) );
if (fd<0) return fd;
ssize_t block = (ssize_t)sizeof(typename M::T_numtype) * data.size();
//WHENTESTING( printf( "fd: %d..\n", fd ) );
ssize_t writ = write( fd, data.data(), block );
close(fd);
//WHENTESTING( printf( "wrote %ld of %ld bytes..\n", (long)writ, (long)block ) );
return writ==block? 0 : -1;
}



template< typename V >
size_t dimensions( V &out, const char *file )
{ /* return the dimensions on the filename in major-first order */
/* e.g. foo.real4.45.23 -> [ 23, 45 ] */
if (!file || !*file) return 1; 
size_t nelems = dimensions<V>( out, strchr( file+1, '.' ) );
if ( (file[0]=='.') && isdigit(file[1]) )
{
    int dim = atoi(file+1);
    out.push_back( dim );
    nelems *= (size_t)dim;
}
return nelems;
}

}; // namespace


#endif

