/** 
*  \file Connector.h
*  \brief landing point for connector-related definitions
*  \author R.K.Garcia <rayg@ssec.wisc.edu>
*
*  \version $Id: Connector.h,v 1.22 2004/10/31 05:07:39 maciek Exp $
*
*
*  Copyright UW/SSEC, ALL RIGHTS RESERVED, 2004
*  Space Science and Engineering Center
*  University of Wisconsin - Madison, USA
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*  \endverbatim
*/

/**
* \mainpage UML-RT connectors implemented the C++ Template-fu way
 *
 * \par SimpleConnector diagram
 * The following diagram shows how the simple connector (whose payload, input and output
 * data structures are one and the same, set at compile time) is built up from its component
 * templated interfaces.
 * \image html simpleconnector.jpg
 *
 * \par PoliciedConnector diagram
 * The policied connector is a bit more complex, allowing an underlying payload type to 
 * be adapted variously for input and output, using different adaptation policies.
 * \image html policiedconnector.jpg
 *
 */


#ifndef H_CONNECTOR
#define H_CONNECTOR

#ifndef NO_DEPENDENCY_INCLUDES

#include "ProtocolRole.h"

#include <iostream>

#endif

///  \brief Connector namespace, including basic policies
namespace Connector 
{ 
    
    //
    // Basic Policies
    //
    
    // Just store the payload as payload_ protected member. 
    template< typename P > class SimplePayload
    { 
    protected: 
        P payload_; 
    public:
        typedef P PayloadType;
        virtual ~SimplePayload< P >( ) 
        { }
    };
    
    
    // upstream side functions as output location for previous operator
    template< typename P, typename B > 
    class MakesPayloadAvailableForOutput: 
        public B, public OutputPort< P > 
    {       
    public:
        // implementation of OutputPort<P>
        virtual operator P &()
        { return payload_; }
    };
    
    
    /** \brief Makes payload_ available directly through InputPort<Payload> interface
     */
    template< typename P, typename B > 
    class MakesPayloadAvailableForInput: 
        public B, public InputPort< P > 
    {
    public:
        // implementation of InputPort<U>
        virtual operator const P &()
        { return payload_; }
    };


    //
    // Slightly more complex payload_ policies
    //

    /** \brief Invoke a payload adapter to allow upstream access
    * Example: blitz array with a different stride, or a view of an array
    * U must be able to initialize from a B::PayloadType
    * provides OutputPort< U::PayloadType >
     */        
    template< class U, class B > 
    class AdaptsPayloadForOutput: 
        public B, public OutputPort< typename U::PayloadType > 
    {
        U payloadAsU_;
    public:
        
        AdaptsPayloadForOutput<U,B>() : payloadAsU_( payload_ )
        { }
                
        // implementation of InputPort<U>
        virtual operator typename U::PayloadType &()
        { return payloadAsU_; }
    };
    
    
    /** \brief Invoke a payload adapter to allow downstream access
    * Example: blitz array with a different stride, or a view of an array
    * D must be able to initialize from a B::PayloadType
    * provides OutputPort< D::PayloadType >
    */        
    template< class D, class B > 
    class AdaptsPayloadForInput: 
        public B, public InputPort< typename D::PayloadType > 
    {
        D payloadAsD_;
    public:
        
        AdaptsPayloadForInput<D,B>() : payloadAsD_( payload_ )
        { }
        
        // implementation of InputPort<U>
        virtual operator const typename D::PayloadType &()
        { return payloadAsD_; }
    };
    
    /** \brief copy the payload to an adapter container for downstream use
    *  This copies the payload every time that the InputPort<D> is accessed!
    *  \todo decide if this should have a copy command usable by control logic
    */
    template< class D, class B > 
    class CopiesPayloadForInput: 
        public B, public InputPort< D > 
    {
        // secondary payload area which contains the translated/reordered data
        // example would be converting from C indexing to Fortran indexing
        // or changing from double to float..
        D payloadCopiedAsD_;
    public:
        
        CopiesPayloadForInput<D,B>() : payloadCopiedAsD_( payload_.size() )
        { }
        
        virtual ~CopiesPayloadForInput<D,B>() 
        { }
        
        // implementation of InputPort<U>
        
        // upon request for input buffer, operator receives a copy of the 
        // payload, with any attendant modifications or restructuring
        virtual operator const D &()
        { 
            // resize payloadCopiedAsD_ to be same size as payload_
            
            // copy content, changing double to float for instance
            // may need a functor template parameter if we want 
            // to do something like complex to real
            std::copy( payload_.begin(), payload_.end(), payloadCopiedAsD_.begin() );
            return payloadAsD_; 
        }
    };
    
    //
    // Connector Templates
    // 

    /**
    ** Simple Connector
     ** example: SimpleConnector< NonlinearSpectrum > point2point;
     **/
    template<
        typename PayloadType
        >
    class SimpleConnector: 
        public MakesPayloadAvailableForInput < PayloadType, 
                    MakesPayloadAvailableForOutput < PayloadType,
                        SimplePayload< PayloadType > > >
    {
    };
    
    /**
    * BlitzSimpleConnector is like a simple connector, except for blitz vectors.
    */
    template< typename ELT_TYPE>
        class BlitzSimpleConnector:
            public BlitzInputPort<ELT_TYPE>, public BlitzOutputPort<ELT_TYPE>
    {
private:
        blitz::Array< ELT_TYPE, 1> backingVector;
public:
        BlitzSimpleConnector( int size ): backingVector( size ) {};
        virtual ~BlitzSimpleConnector() {};
        virtual blitz::Array< ELT_TYPE, 1 > getForOutput() { return backingVector( blitz::Range::all() ); }
        virtual const blitz::Array< ELT_TYPE, 1 > getForInput() { return backingVector( blitz::Range::all() ); }
    }; // class BlitzSimpleConnector
        
        
    
    /// \class BlitzMatrixBackedInputConnector
    ///
    /// \brief uses a blitz memory backing (with multiple records) to provide input
    /// 
    /// The idea is to provide an input connector which is backed by a predefined memory
    /// buffer - in this case, a two-dimensional blitz::Array, representing a set of
    /// records, each record being a one-dimensional blitz::Array payload.
    /// \par
    /// seek and next methods provide the controlling code the ability to select which
    /// record is the designated payload.
    /// \par
    /// the intended uses of this class are threefold:
    /// 
    /// - for testing, a set of blitz records may be read into memory, say from a file,
    /// and the connector may be used for iterating (via next) through them
    /// - accessing memory-mapped binary files, such as FBF files.
    /// - double (or multi)-buffering schemes, where memory for two (or more) records is
    /// allocated, and while one record is being used as a payload through the connector,
    /// another record may be being externally updated by another thread. (This use will need
    /// more thread-safety than is provided by the current prototype implementation)
    ///
    /// Note that while multiple threads can access a common memory area with their own
    /// BlitzMatrixBackedInputConnectors, a single connector cannot be handed among
    /// multiple threads.
    
    template< typename ELT_TYPE > 
        class BlitzMatrixBackedInputConnector: public BlitzInputPort<ELT_TYPE>
    {
private:
        const blitz::Array<ELT_TYPE,2> &backingMatrix;
        int currentRecord;
        const int numRecords;

public:
        BlitzMatrixBackedInputConnector( const blitz::Array<ELT_TYPE,2> &bm): 
            backingMatrix(bm),
            currentRecord(0),
            numRecords( backingMatrix.shape()(0) )
        {}; ///< ctor
        

        virtual ~BlitzMatrixBackedInputConnector() { }; ///< dtor
        
        int seek( const int index ) { return currentRecord = index; }; ///< changes payload record number
        int tell() { return currentRecord; }; ///< returns currently selected payload record number
        int next()
        { 
            currentRecord = ( currentRecord + 1 ) % numRecords; 
            return currentRecord;
        }; ///<increments payload record number. Loops back to 0 when reaching end of records.

        int getNumRecords() { return numRecords; };
        /// implementation of InputPort
        virtual const blitz::Array<ELT_TYPE,1> getForInput() 
        { 
            return backingMatrix( currentRecord, blitz::Range::all() );
        };
    };
    
    

    /// and the corresponding output port. Like input, but without the const.
    template< typename ELT_TYPE >
        class BlitzMatrixBackedOutputConnector: public BlitzOutputPort<ELT_TYPE>
    {
private:
            blitz::Array<ELT_TYPE, 2> &backingMatrix;
            int currentRecord;
            const int numRecords;

public:
            BlitzMatrixBackedOutputConnector( blitz::Array<ELT_TYPE, 2> &bm):
                backingMatrix(bm),
                currentRecord(0),
                numRecords( backingMatrix.shape()(0) )
            {}; ///< ctor
            
            virtual ~BlitzMatrixBackedOutputConnector() { }; ///< dtor
            int next()
            { 
                currentRecord = ( currentRecord + 1 ) % numRecords; 
                return currentRecord;
            }; ///<increments payload record number. Loops back to 0 when reaching end of records.
            
            int getNumRecords() { return numRecords; };
            
            /// implementation of outputPort
            virtual blitz::Array< ELT_TYPE, 1 > getForOutput() 
            { 
                return backingMatrix( currentRecord, blitz::Range::all() );
            };
    };
    
    /**
    ** \brief Connector With Upstream and Downstream Policies 
    ** Example: 
     ** PoliciedConnector< NonlinearSpectrum, NonlinearSpectrum, LinearSpectrum >
     **   nonlinearityCorrectionBypassConnector; 
     **/
    template< 
        typename UpstreamType,
        typename PayloadType,
        typename DownstreamType,
        template< class, class > class UpstreamPolicy 
            = AdaptsPayloadForOutput,
        template< class, class > class DownstreamPolicy 
            = AdaptsPayloadForInput,
        template< class > class PayloadPolicy 
            = SimplePayload
        > 
    class PoliciedConnector: 
        public UpstreamPolicy< UpstreamType,  
                 DownstreamPolicy< DownstreamType,
                    PayloadPolicy< PayloadType > > >
    {
    };
    
    
    /// here, we create a template that properly constructs a blitz array 
    /// referring to the content of a std vector. 
    /// note, this is dangerous in the case that the vector changes size! 
    /// specialization of blitz array A using an underlying in-memory block B
    ///
    /// \deprecated Use BlitzMatrixBackedInputConnector instead
    template< class A, class B >
        class BlitzArrayFromMemoryBuffer : public A
    {
public:
        // we need to provide a clean constructor in order to be able to adapt
        BlitzArrayFromMemoryBuffer<A,B> ( B &it ):
        A( &it[0], blitz::shape( it.size() ), blitz::neverDeleteData )
    { }
        
        typedef A PayloadType;
    }; // class BlitzArrayFromMemoryBuffer
    
    
    
    /** a port wired to always send the same data - useful for initialization settings,
        guaranteed not to change in the course of runtime */
    template< class Payload >
        class ConstantInputPort: public InputPort< Payload >
    {
        Payload payload_;
public:
        ConstantInputPort<Payload>( const Payload &payload ): 
        payload_(payload) 
    { };
        virtual ~ConstantInputPort<Payload>() { };
        
        virtual operator const Payload & ( ) { return payload_; }; 
    }; // class ConstantInputPort
    
    
    
    
    /** lookup variant of the ConstantInputPort - again, useful for settings at initialization, 
        guaranteed not to change in the course of runtime */
    template< class Key, class Payload >
        class ConstantLookupInputPort: public LookupInputPort< Key, Payload >
    {
        Payload payload_;
public:
        ConstantLookupInputPort<Key,Payload>( const Payload &payload ): 
        payload_(payload) 
    { }
        virtual ~ConstantLookupInputPort<Key,Payload>() { }
        
        // implementation of LookupInputPort<K,P>
        virtual const Payload &operator() ( const Key & ) { return payload_; } 
    }; // class TrivialLookupInputPort

    
    
    
    
}; // namespace

#endif // ifndef H_CONNECTOR









