/** 
*  \file ConnectorTest.cc
*  \brief unit test for Connector library
*
*  \author R.K.Garcia <rayg@ssec.wisc.edu>
*
*  \version $Id: ConnectorTest.cc,v 1.4 2004/10/29 16:22:13 rayg Exp $
*
*
*  Copyright UW/SSEC, ALL RIGHTS RESERVED, 2004
*  Space Science and Engineering Center
*  University of Wisconsin - Madison, USA
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*  \endverbatim
*/

/*
 ln -s Connector.h test1.cc
 c++ -DTEST=1 -Wall -Wno-long-double -I../Common -I/sw/include -c test1.cc 2>&1 | less
 c++ -O3 -DTEST=1 -Wall -Wno-long-double -L/sw/lib -I../Common -I/sw/include ConnectorTest.cc -o ConnectorTest.unittest -lblitz -lstdc++
 */


#include <complex>
#include <vector>
#include <blitz/array.h>

#include "Connector.h"
#include "TestConnector.h"

using namespace Connector;
using namespace TestConnector;




// this is our storage class, which below we declare to be compatible 
// with the nonlinear and linear spectra
typedef std::complex<double> complex_double;

typedef blitz::Array<complex_double,1> NonlinearSpectrum;
typedef blitz::Array<complex_double,1> LinearSpectrum;


typedef blitz::Array< complex_double, 1 > NonlinearSpectrum;

// simple test case
SimpleConnector< NonlinearSpectrum > point2point;



template< class A, class B >
class BlitzArrayFromCompatibleBlitzArray : public A
{
public:
    // we need to provide a clean constructor in order to be able to adapt
    // also, technically in err if source array can resize
    BlitzArrayFromCompatibleBlitzArray<A,B> ( B &it ):
    A( it.data(), it.shape(), blitz::neverDeleteData )
    { }
    
    // since we are an adapter, we have a payload which looks like this
    typedef A PayloadType;
};



// a non-resizable vector; might be able to reuse std::vector to do this
template< typename T, int Size >
class FixedMemoryBuffer
{
    T data_[Size];
public:
    inline operator T*( ) { return data_; }
    inline int size() const { return Size; }
};


typedef FixedMemoryBuffer< complex_double, 1024 > ComplexLongwaveStore;

typedef BlitzArrayFromMemoryBuffer< NonlinearSpectrum, ComplexLongwaveStore > NonlinearSpectrumFromLongwaveStore;
typedef BlitzArrayFromMemoryBuffer< LinearSpectrum, ComplexLongwaveStore > LinearSpectrumFromLongwaveStore;
  

//
// and after all that kicking and punching, we get the simple syntax.. 
//  

PoliciedConnector< 
        NonlinearSpectrumFromLongwaveStore, 
        ComplexLongwaveStore, 
        LinearSpectrumFromLongwaveStore > 
    nonlinearityCorrectionBypassConnector;


//
// Let's try something more in the middle ground, where you don't have 
// an adapter on both sides. Technically a little more dangerous because 
// sizing isn't pinned at compile time.
//

PoliciedConnector< 
    NonlinearSpectrum,
    NonlinearSpectrum,
    BlitzArrayFromCompatibleBlitzArray< LinearSpectrum, NonlinearSpectrum >,
    MakesPayloadAvailableForOutput 
    // default the rest to using adapter and simple payload
    > alternateNonlinearityCorrectionBypassConnector;


// 
// But! This gives us a set of tools for doing the next set of connectors 
//   in possibly even less time than that one.. 
//  This begins to feel somewhat like piecing together aspects.. 
//



void test1()
{
    OutputPort<NonlinearSpectrum> &upstream( nonlinearityCorrectionBypassConnector );
    InputPort<LinearSpectrum> &downstream( nonlinearityCorrectionBypassConnector );
    NonlinearSpectrum &inside( upstream );
    const LinearSpectrum &outside( downstream );
    
    assert( inside.size() == outside.size() );
    
    inside = complex_double(114,12);
    
    assert( outside(1022) == complex_double(114,12) );
}    

void test2()
{    
    blitz::Array< complex_double, 2 > buffer( 32, 1 ); // 32 records by 15 values per record
    BlitzMatrixBackedOutputConnector< complex_double > ocon( buffer );
    
    blitz::Array<complex_double,1> &oar( ocon );
    oar(0) = 1;
    assert( buffer(0,0) == complex_double(1) );
    
    ocon.next();
    assert( buffer(0,0) == complex_double(1) );

    oar(0) = 2;
    assert( buffer(1,0) == complex_double(2) );
}    


#include <cstdlib>
int main() 
{
    test1();
    test2();
    exit(0);
}


