/**
 * \file CplxFFT.h
 * \brief performs complex FFT
 * \author Maciek Smuga-Otto <maciek@ssec.wisc.edu>
 *
 * based directly on the Scanning-HIS rsh2fbf application, written by Ray Garcia <rayg@ssec.wisc.edu>
 *
 *  \version $Id: CplxFFT.h,v 1.11 2004/10/31 05:07:39 maciek Exp $
 *
 *  \par Copyright:
 *  \verbatim
 *
 *  Copyright UW/SSEC, ALL RIGHTS RESERVED, 2004
 *  Space Science and Engineering Center
 *  University of Wisconsin - Madison, USA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *  \endverbatim
 */

#ifndef H_CPLXFFT
#define H_CPLXFFT

#ifndef NO_DEPENDENCY_INCLUDES

// the FFTW system-dependent #include polka
#ifdef __APPLE__
#include "dfftw_threads.h"
#include "drfftw_threads.h"
#elif defined(__CYGWIN32__)
#include "fftw.h"
#include "rfftw.h"
#define fftw_threads_init() 
#define fftw_threads_one(a,b,c,d) fftw_one(b,c,d)
#define rfftw_threads_one(a,b,c,d) rfftw_one(b,c,d)
#else
#include "fftw_threads.h"
#include "rfftw_threads.h"
#endif

// GIPS includes
#include "Physics.h"
#include "Instrument.h"
#include "ProtocolRole.h"
#include "Operator.h"

#endif

namespace CplxFFT 
{    
    /// \brief Config_t is the initialization-time configuration.  
    struct Config_t {
        size_t npts; ///< number of points in spectrum
        explicit Config_t( size_t n ): npts( n ) { };
    }; // struct Config_t
    
    typedef  Connector::BlitzInputPort< std::complex< double > > &JackIn;
    typedef  Connector::BlitzOutputPort< std::complex< double > > &JackOut;
    typedef  Connector::InputPort< Config_t > &JackConfig;
    
    /// \brief Ports for the Complex FFT operator.
    struct Ports {
        JackIn shiftedInterferogram; ///< input interferogram, correctly shifted for FFT
        JackOut prefoldedSpectrum; ///< output spectrum, not yet re-folded
        JackConfig config; ///< configuration settings 
        
        Ports( JackIn i, JackOut o, JackConfig c ): 
            shiftedInterferogram(i), prefoldedSpectrum(o), config(c) 
        {}; ///< ctor
        ~Ports() {}; ///< dtor
    };
 
    /// \brief performs complex FFT from raw interferogram to uncorrected, uncalibrated spectrum.
    ///
    /// uses the FFTW (version 2) library to perform a complex FFT from interferogram to spectral space.
    class CplxFFTOperator: Operator::AbstractOperator
    {
private:
        fftw_plan plan; // cached machine-dependent plan
        Ports &ports;
public:
        explicit CplxFFTOperator( Ports & ); ///< ctor
        virtual ~CplxFFTOperator(); ///< dtor
        virtual Operator::Result operator() ( ); ///< make it so
    }; // class Operator
    
    
} // namespace
#endif

