/*
 * \file SHIS_Ifg2SpectrumTest.cc
 * \brief unit test for SHISStage subclass of Ifg2SpectrumStage
 * \author Maciek Smuga-Otto <maciek@ssec.wisc.edu>
 *
 *  \version $Id: SHIS_Ifg2SpectrumTest.cc,v 1.13 2004/10/31 05:07:39 maciek Exp $
 *
 *  \par Copyright:
 *  \verbatim
 *
 *  Copyright UW/SSEC, ALL RIGHTS RESERVED, 2004
 *  Space Science and Engineering Center
 *  University of Wisconsin - Madison, USA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 *  \endverbatim
 */

// how to test the nonlinearity correction operator?
// for one thing, we need some ersatz connectors that provide
// data from a compile-time allocated memory cache, and read back
// into another one (at least for the unit tests)


// legacy C includes
#include <cstdlib> // exit
#include <cassert> // assert
//#include <cstring> // memset

// C++ STL includes
#include <string>
#include <vector>
#include <iostream> // std::cerr

// other support library includes
#include "FBF.h"
#include "clok.h"

// GIPS framework includes
#include "Ifg2Spectrum.h"
#include "TestConnector.h"

using namespace std;

namespace Ifg2Spectrum
{
    
Operator::Result livelinessTest( )
{

    // data in & out
    blitz::Array< complex<double>, 2> blitzIn( 1, 1024 );
    Connector::BlitzMatrixBackedInputConnector< complex<double> > testIn( blitzIn );
    
    blitz::Array< complex<double>, 2> blitzOut( 1, 1024 );
    Connector::BlitzMatrixBackedOutputConnector< complex<double> > testOut( blitzOut );
    
    // configuration - used at initialization of stage and operators
    Config_t defaultConfig( 1024, 512 );
    Connector::ConstantInputPort< Config_t > testConfig( defaultConfig );
    
    Ports ports( testIn, testOut, testConfig );

    Ifg2SpectrumStage *stageptr = Ifg2SpectrumStage::Factory( Instrument::SHIS, ports );
    //Ifg2SpectrumStage &stage( *stageptr ); // local name to avoid using pointer syntax
    Operator::Result res = stageptr->operator()( );
    
    return res;
}; // testOperator()

Operator::Result spectrumFileTest(const char *INPUT_FILENAME, const char *OUTPUT_FILENAME) 
{
    // getting at the input file
    vector< complex<double> > *rawInput = new vector< complex<double> >(0);
    FILE *infp = fopen( INPUT_FILENAME, "rb" );
    FBF::load( *rawInput, infp );
    fclose( infp );
    
    const blitz::Array< complex<double>, 2> blitzIn(&(*rawInput)[0], blitz::shape(1000,2044), blitz::deleteDataWhenDone);
    Connector::BlitzMatrixBackedInputConnector< complex<double> > testIn( blitzIn );
    
    // preparing the output buffer
    //blitz::Array< complex<double>, 2 > *blitzOut = new blitz::Array< complex<double>, 2 >(blitz::shape(1000,2044));
    blitz::Array< complex<double>, 2 > blitzOut(blitz::shape(1000,2044));
    Connector::BlitzMatrixBackedOutputConnector< complex<double> > testOut( blitzOut );
    
    // and the test settings
    Config_t defaultConfig( 2044, 829 );
    Connector::ConstantInputPort< Config_t > testConfig( defaultConfig );
    
    // set up the run
    Ports ports( testIn, testOut, testConfig );
    Ifg2SpectrumStage *stageptr = Ifg2SpectrumStage::Factory( Instrument::SHIS, ports );
    
    
    // run!
    Operator::Result res;
    Clok timer;
    std::cerr << "data loaded. starting main loop \n";
    timer.start();
    for (int i = 0; i < 1000; ++i)
    {
        res = stageptr->operator()( );
        assert (res == Operator::OK);
        testIn.next();
        testOut.next();
        // std::cerr << "output channels 0,10: " << blitzOut(0, 0) << "," << blitzOut(0, 10) << "\n";
        
    }
    timer.stop();
    std::cerr << "finished processing. Time stats: " << timer.report() << "\nsaving result\n";
    
    if (res == Operator::OK) {
        // and write the results to file
        int stat = FBF::blitz_save( OUTPUT_FILENAME, blitzOut );
        if (stat != 0 )
            res = Operator::ERROR;
    }
    
    return res;
};

}; // namespace


int main( int argc, char *argv[] )
{
    Operator::Result rc;
    
    if (argc == 3) // gave input, output filenames as arguments
    {
        rc = Ifg2Spectrum::spectrumFileTest(argv[1], argv[2]);
        assert (rc==Operator::OK);
    }
    else
    {
            rc = Ifg2Spectrum::livelinessTest();
            assert( rc==Operator::OK );
    }
        
    
    exit(0);
}

