/**
 * \file SpectrumFold.h
 * \brief transforms spectrum around a fold point.
 *
 * Necessary after complex FFT from raw interferogram.
 * \author Maciek Smuga-Otto <maciek@ssec.wisc.edu>
 *
 * based directly on the Scanning-HIS rsh2fbf application, written by Ray Garcia <rayg@ssec.wisc.edu>
 *
 *  \version $Id: SpectrumFold.h,v 1.12 2004/10/31 05:07:39 maciek Exp $
 *
 *  \par Copyright:
 *  \verbatim
 *
 *  Copyright UW/SSEC, ALL RIGHTS RESERVED, 2004
 *  Space Science and Engineering Center
 *  University of Wisconsin - Madison, USA
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  \endverbatim
 */

#ifndef H_SPECFOLD
#define H_SPECFOLD

#ifndef NO_DEPENDENCY_INCLUDES
// GIPS includes
#include "Physics.h"
#include "Instrument.h"
#include "ProtocolRole.h"
#include "Operator.h"

#endif

namespace SpecFold 
{    
    /// Config_t holds initialization-time configuration settings.
    
    struct Config_t {
        size_t npts;  ///< number of points in the spectrum (length of vector)
        int foldPoint; ///< fold point around which spectrum is transformed
        Config_t( size_t n, int f ): npts( n ), foldPoint( f ) { }; ///< ctor
    };

    typedef  Connector::BlitzInputPort< std::complex< double > > &JackIn;
    typedef  Connector::BlitzOutputPort< std::complex< double > > &JackOut;
    typedef  Connector::InputPort< Config_t > &JackConfig;
    
    /// Ports is the port structure for the operator.
    struct Ports {
        JackIn prefoldedSpectrum;
        JackOut foldedSpectrum;
        JackConfig config;
        
        Ports( JackIn i, JackOut o, JackConfig c): 
            prefoldedSpectrum(i), foldedSpectrum(o), config(c)
        {};
        ~Ports() {};
    };
    
    /// \brief concrete operator which performs the spectral folding
    /// 
    /// transforms an original vector
    /// v_0 ... v_pf v_pf+1 ... v_npts-1
    /// to the vector
    /// v_pf ... v_0 v_npts-1 ... v_pf+1
    /// where pf = npts - foldpoint
    /// by flipping the two segments split at pf.
    class SpecFoldOperator: Operator::AbstractOperator
    {
private:
        Ports &ports;
public:
        explicit SpecFoldOperator( Ports &p ): ports(p) { }; ///< ctor
        virtual ~SpecFoldOperator() { }; ///< dtor
        virtual Operator::Result operator() ( ); ///< make it so
    }; // class Operator
    
    
} // namespace
#endif

